#include "CommandLib.h"
#include <stdlib.h>


quint8 Div_4(quint32 data,quint8 index);
quint8 Div_8(quint32 data,quint8 index);

int ascii2hex(quint8 ch);

quint8 int2ascii(quint32 val , unsigned char *SourceBuffer, quint8 size);
quint32 ascii2int(quint8 *SourceBuffer, quint8 size);

quint8 hex2ID(quint32 val,quint8 idCount, quint8 *SourceBuffer, quint8 size); //ID(hex) --> Ascii(ڿ)
quint32 ascii2ID_Rev(quint8 *SourceBuffer, quint8 idCount); //ڿ --> id

quint8 toASCII_HEX(quint8 int_hex); //hex-->ascii

bool IsSTD(quint8 format){
	if((format & 0x06) == 0x04)
		return true;
	else 
		return false;
}
bool IsEXT(quint8 format){
	if((format & 0x06) == 0x06)
		return true;
	else 
		return false;
}
bool IsDATA(quint8 format){
	if((format & 0x05) == 0x04)
		return true;
	else 
		return false;
}
bool IsREMOTE(quint8 format){
	if((format & 0x05) == 0x05)
		return true;
	else 
		return false;
}



quint8 SendCANTxFrame(CAN_Struct Tx, quint8 * SerialBuffer, quint8 BufferSize, quint8 * ConvertSize)
{
	int i;
	quint8 ind=0;
	quint8 tmp_index;
	quint8 buf[40];

	switch (Tx.Format)
	{
	case CAN_StructFormat::TX_STD_DATA: buf[ind++] = CAN_SerialCommandHeader::STD_DATA;	break;
	case CAN_StructFormat::TX_STD_REMOTE: buf[ind++] = CAN_SerialCommandHeader::STD_REMOTE;	break;
	case CAN_StructFormat::TX_EXT_DATA: buf[ind++] = CAN_SerialCommandHeader::EXT_DATA;	break;
	case CAN_StructFormat::TX_EXT_REMOTE: buf[ind++] = CAN_SerialCommandHeader::EXT_REMOTE;	break;
	default: return Invalid_Arg;
	}

	if(IsSTD(Tx.Format))
		tmp_index = 3;
	else if(IsEXT(Tx.Format))
		tmp_index = 8;

	for(i=1;i<=tmp_index;i++){
		buf[ind++] = toASCII_HEX(Div_4(Tx.ID,tmp_index - i));
	}

	buf[ind++] = toASCII_HEX(Tx.DLC);
	
	if(IsDATA(Tx.Format)){

		for(i=0;i<Tx.DLC;i++){
			buf[ind++] = toASCII_HEX(Div_4(Tx.DATA[i],1));
			buf[ind++] = toASCII_HEX(Div_4(Tx.DATA[i],0));
		}
	}

	buf[ind++] = CR;
	
	if(ind > BufferSize)
		return Invalid_Arg;

	*ConvertSize = ind;

	memcpy(SerialBuffer,buf,ind);
	
	return No_Error;

}


quint8 SetSerialConfig(SerialConfigInfo info, quint8 * SerialBuffer, quint8 BufferSize, quint8 * ConvertSize){
	
	quint8 len;
	unsigned char buf[20];

	buf[0] = 'W';
	buf[1] = 'S';
	switch (info.flow){
	case NoFlowControl:  buf[2] = 'N'; break;
	case HardwareControl: buf[2] = 'H'; break;
	default: return Invalid_Arg;
	}

	switch (info.data){
	case Data8:  buf[3] = '8'; break;
	default: return Invalid_Arg;
	}

	switch (info.parity){
	case NoParity:		buf[4] = 'N'; break;
	case EvenParity:	buf[4] = 'E'; break;
	case OddParity:		buf[4] = 'O'; break;
	case SpaceParity:	buf[4] = 'S'; break;
	case MarkParity:	buf[4] = 'M'; break;
	default: return Invalid_Arg;
	}

	switch (info.stop){
	case OneStop:  buf[5] = '1'; break;
	case TwoStop:  buf[5] = '2'; break;
	default: return Invalid_Arg;
	}
	len = int2ascii(info.Baudrate,&buf[6],20-6);
	if(len ==0 ){
		return Invalid_Arg;
	}
	buf[6+len] = CR;


	if((len+7) > BufferSize)
		return Invalid_Arg;

	*ConvertSize = len+7;
	memcpy(SerialBuffer,buf,len+7);

	return No_Error;


}
quint8 SetCANConfig(CANConfigInfo info, quint8 * SerialBuffer, quint8 BufferSize, quint8 * ConvertSize){
	quint8 len;
	quint8 ind;
	unsigned char buf[30];
	quint8 function;
	function =0;


	buf[0] = 'W';
	buf[1] = 'C';
	switch (info.Spec){
	case CAN_A: buf[2] = 'A'; break;
	case CAN_B: buf[2] = 'B'; break;
	default: return Invalid_Arg;
	}
	buf[3] = ',';

	len = int2ascii(info.Baudrate,&buf[4],4);
	if(len ==0 ){
		return Invalid_Arg;
	}
	buf[len+4] = ',';
	ind=len+5;

	len = hex2ID(info.ID,info.Spec,&buf[ind],8);
	if(len ==0 ){
		return Invalid_Arg;
	}
	buf[len+ind] = ',';
	ind=len+ind+1;

	len = hex2ID(info.Mask,info.Spec,&buf[ind],8);
	if(len ==0 ){
		return Invalid_Arg;
	}
	buf[len+ind] = ',';
	ind=len+ind+1;

	if(info.DAR)
		SETDAR(function);
	if(info.ABOR)
		SETABOR(function);

	buf[ind++] = toASCII_HEX(function);
	buf[ind++] = CR;


	if(ind > BufferSize)
		return Invalid_Arg;
	
	*ConvertSize = ind;
	memcpy(SerialBuffer,buf,ind);
	

	return No_Error;

}


quint8 GetSerialConfig(quint8 * SerialBuffer, quint8 BufferSize, quint8 * ConvertSize){
	unsigned char buf[5];
	buf[0] = 'R';
	buf[1] = 'S';
	buf[2] = CR;
	if(3 > BufferSize)
		return Invalid_Arg;
	*ConvertSize = 3;
	memcpy(SerialBuffer,buf,3);
	return No_Error;
}

quint8 GetCANConfig(quint8 * SerialBuffer, quint8 BufferSize, quint8 * ConvertSize){

	unsigned char buf[5];
	buf[0] = 'R';
	buf[1] = 'C';
	buf[2] = CR;
	if(3 > BufferSize)
		return Invalid_Arg;
	*ConvertSize = 3;
	memcpy(SerialBuffer,buf,3);
	return No_Error;
}
quint8 SaveConfig(quint8 * SerialBuffer, quint8 BufferSize, quint8 * ConvertSize){
	unsigned char buf[5];
	buf[0] = 'S';
	buf[1] = 'V';
	buf[2] = CR;
	if(3 > BufferSize)
		return Invalid_Arg;
	*ConvertSize = 3;
	memcpy(SerialBuffer,buf,3);
	return No_Error;
}
quint8 FirmwareUpgrade(quint8 * SerialBuffer, quint8 BufferSize, quint8 * ConvertSize){
	unsigned char buf[5];
	buf[0] = 'F';
	buf[1] = 'U';
	buf[2] = CR;
	if(3 > BufferSize)
		return Invalid_Arg;
	*ConvertSize = 3;
	memcpy(SerialBuffer,buf,3);
	return No_Error;
}
quint8 FirmwareVersion(quint8 * SerialBuffer, quint8 BufferSize, quint8 * ConvertSize){
	unsigned char buf[3];
	buf[0] = 'F';
	buf[1] = 'V';
	buf[2] = CR;
	if(3 > BufferSize)
		return Invalid_Arg;
	*ConvertSize = 3;
	memcpy(SerialBuffer,buf,3);
	return No_Error;
}
quint8 GetErrorInfo(quint8 * SerialBuffer, quint8 BufferSize, quint8 * ConvertSize){
	unsigned char buf[3];
	buf[0] = '!';
	if(1 > BufferSize)
		return Invalid_Arg;
	*ConvertSize = 1;
	memcpy(SerialBuffer,buf,1);
	return No_Error;
}



quint8 ParsingSerialInfo(SerialConfigInfo * info, quint8 * SerialBuffer, quint8 BufferSize){
	SerialConfigInfo tmp;
	quint32 baudrate;
	quint8 flow, stop, data, parity;

	flow = SerialBuffer[0];
	data = SerialBuffer[1];
	parity = SerialBuffer[2];
	stop = SerialBuffer[3];
	baudrate = ascii2int(&SerialBuffer[4], BufferSize-5);

	switch (flow)
	{
	case 'N': tmp.flow = NoFlowControl; break;
	case 'H': tmp.flow = HardwareControl; break;
	default: return Invalid_Arg;
	}

	switch (data)
	{
	case '8': tmp.data = Data8; break;
	default: return Invalid_Arg;
	}

	switch (parity)
	{
	case 'N': tmp.parity = NoParity; break;
	case 'E': tmp.parity = EvenParity; break;
	case 'O': tmp.parity = OddParity; break;
	case 'S': tmp.parity = SpaceParity; break;
	case 'M': tmp.parity = MarkParity; break;
	default: return Invalid_Arg;
	}
	switch (stop)
	{
	case '1': tmp.stop = OneStop; break;
	case '2': tmp.stop = TwoStop; break;
	default: return Invalid_Arg;
	}
	if(baudrate%100)
		return Invalid_Arg;
	else{
		tmp.Baudrate = baudrate;
	}

	//*info = tmp;
	memcpy(info,&tmp,sizeof(SerialConfigInfo));
	return No_Error;
}
quint8 ParsingCANInfo(CANConfigInfo * info, quint8 * SerialBuffer, quint8 BufferSize){
	CANConfigInfo tmp;
	char org[50];
	char *ptr;
	quint8 tt[5][10];
	quint8 i;
	quint8 function;
	
	switch(SerialBuffer[0])
	{
	case 'A':
		tmp.Spec = CAN_A;
		if(14 >= BufferSize ||  BufferSize >= 18)
			return Invalid_Arg;
		break;
	case 'B':
		if(24 >= BufferSize ||  BufferSize >= 28)
			return Invalid_Arg;
		tmp.Spec = CAN_B;
		break;
	default: return Invalid_Arg;
	}

	memcpy(org,SerialBuffer,BufferSize);
	memset(tt,0,5*10);
	ptr = strtok(org,",");
	i=0;
	while(ptr=strtok( NULL, ",")){
		memcpy(tt[i],ptr,strlen(ptr));
		i++;
	}

	tmp.Baudrate = ascii2int(tt[0],strlen((char*)tt[0]));
	tmp.ID = ascii2ID_Rev(tt[1],tmp.Spec);
	tmp.Mask = ascii2ID_Rev(tt[2],tmp.Spec);
	function = ascii2int(tt[3], 1);

	if(function > 255)
		return No_Error;

	if(ISDAR(function))
		tmp.DAR = true;
	else
		tmp.DAR = false;

	if(ISABOR(function))
		tmp.ABOR = true;
	else
		tmp.ABOR = false;

	//*info = tmp;
	memcpy(info,&tmp,sizeof(CANConfigInfo));
	return No_Error;

}

quint8 ParsingErrorInfo(CANErorrInfo * info, quint8 * SerialBuffer, quint8 BufferSize){
	CANErorrInfo tmp;
	char org[50];
	char *ptr;
	quint8 tt[5][10];
	
	
	memcpy(info,&tmp,sizeof(CANErorrInfo));
	return No_Error;

}




/**********************************************************************

                       private functions 

**********************************************************************/


quint8 Div_8(quint32 data,quint8 index){
	quint8 ret;
	data = data >> (index * 8);
	ret = (data & 0xFF);
	return ret;
}
int ascii2hex(quint8 ch){
	if(0x30 <= ch && ch <= 0x39){//0~9
		ch &= 0x0f;
		return ch;
	}
	else if(0x61 <= ch && ch <= 0x66){ //a~f
		ch &= 0x0f;
		ch += 9;
		return ch;
	}
	else if(0x41 <= ch && ch <= 0x46){ //A~F
		ch &= 0x0f;
		ch += 9;
		return ch;
	}
	else
		return -1;
}

quint32 ascii2ID_Rev(quint8 *SourceBuffer, quint8 idCount){
	quint32 idTmp=0;
	quint8 tmp=0,i=0;
	for(i=0;i<idCount;i++){
		tmp=ascii2hex(SourceBuffer[i]);
		if(tmp!=-1){
			idTmp=idTmp<<4;
			idTmp |= tmp;
		}
	}
	return idTmp;
}



quint8 Div_4(quint32 data,quint8 index){
	quint8 ret;
	data = data >> (index * 4);
	ret = (data & 0x0F);
	return ret;
}
quint8 int2ascii(quint32 val , unsigned char *SourceBuffer, quint8 size){
	quint32 check=10;
	quint8 su=1;

	if(val<10)
		return 0;

	while(val/check){
		su++;
		check*=10;
	}

	if(su>size)
		return 0;

	sprintf((char*)SourceBuffer,"%d",val);
	return su;
}
quint32 ascii2int(quint8 *SourceBuffer, quint8 size){
	quint32 val = 0;
	quint8 ind=0;
	int ret = 0;
	quint32 mul=1;


	while(ret != -1){
		ret = ascii2hex(SourceBuffer[ind++]);
	}
	ind--;
	while(ind){
		ind--;
		ret = ascii2hex(SourceBuffer[ind]);
		val = val + (ret*mul);
		mul*=10;
	}
	return val;
}



quint8 hex2ID(quint32 val,quint8 idCount, quint8 *SourceBuffer, quint8 size){

	quint32 comp;

	switch(idCount){
	case CAN_A: comp = 0x07ff; break;
	case CAN_B: comp = 0x1fffffff; break;
	default: return 0;
	}

	if(val > comp)
		return 0;

	if(idCount > size)
		return 0;

	switch(idCount){
	case CAN_A: sprintf((char*)SourceBuffer,"%03X",val); break;
	case CAN_B: sprintf((char*)SourceBuffer,"%08X",val); break;
	default: return 0;
	}
	return idCount;

}


quint8 toASCII_HEX(quint8 int_hex){
	quint8 ret=0;
	if( 0 <= int_hex && int_hex <= 9){
		ret = '0'+int_hex;
	}else if( 0x0A <= int_hex && int_hex <= 0x0F ){
		ret = 'A'+(int_hex%10);
	}
	return ret;
}