#pragma once
#include <QtCore/QtGlobal>
#include "define.h"


/*************************************
		define  
**************************************/

#define ISDAR(x)	(x)&0x01
#define ISABOR(x)	(x)&0x02

#define SETDAR(x)	(x) = (x)|0x01
#define SETABOR(x)	(x) = (x)|0x02

#define CR 0x0D

//Error Code
#define Invalid_Arg	0x01
#define No_Error	0x00


/*************************************
		CAN Serial Struct 
**************************************/
typedef struct{
	UINT8 Format;
	UINT32 ID;
	UINT8 DLC;
	UINT8 DATA[8];
}CAN_Struct;

enum CAN_StructFormat {
    TX_STD_DATA = 0x14,
    TX_STD_REMOTE = 0x15,
    TX_EXT_DATA = 0x16,
    TX_EXT_REMOTE = 0x17,
	RX_STD_DATA = 0x04,
    RX_STD_REMOTE = 0x05,
    RX_EXT_DATA = 0x06,
    RX_EXT_REMOTE = 0x07
};
/*************************************
		CAN Info Struct 
**************************************/
typedef struct{
	UINT32 Baudrate;
	UINT32 ID;
	UINT32 Mask;
	UINT8 Spec;
	bool DAR;
	bool ABOR;
}CANConfigInfo;

enum CANSpec {
    CAN_A = 3,	//Max Standard CAN ID Length for ascii (0~7FF)
    CAN_B = 8	//Max Extended CAN ID Length for ascii (0~1FFFFFFF)
};
/*************************************
		Option Info Struct 
**************************************/
typedef struct{
	UINT8 mode;
	UINT8 delay;
	UINT8 STD_D_Command_Header;
	UINT8 STD_R_Command_Header;
	UINT8 EXT_D_Command_Header;
	UINT8 EXT_R_Command_Header;
}OptionInfo;

enum CAN_SerialCommandHeader { 
    STD_DATA = 't',
    STD_REMOTE = 'T',
    EXT_DATA = 'e',
    EXT_REMOTE = 'E'
};//CS-CAN Default Serial Command Header


/*************************************
		CAN Error Info Struct 
**************************************/
typedef struct{
	UINT8 TEC;			//Transmit Error Count
	UINT8 REC;			//Receive Error Count
	UINT8 LEC;			//Last Error Count

	UINT32 StuffCount;	//Stuff Error Count
	UINT32 FormCount;	//Form Error Count
	UINT32 AckECount;	//Ack Error Count
	UINT32 BitCount;	//Bit Error Count
	UINT32 CRCCount;	//CRC Error Count

	UINT8 errorInfo;	//Erroe Status
}CANErorrInfo;



quint8 SendCANTxFrame(CAN_Struct Tx, quint8 * SerialBuffer, quint8 BufferSize, quint8 * ConvertSize);


quint8 SetSerialConfig(SerialConfigInfo info, quint8 * SerialBuffer, quint8 BufferSize, quint8 * ConvertSize);
quint8 SetCANConfig(CANConfigInfo info, quint8 * SerialBuffer, quint8 BufferSize, quint8 * ConvertSize);

quint8 GetSerialConfig(quint8 * SerialBuffer, quint8 BufferSize, quint8 * ConvertSize);
quint8 GetCANConfig(quint8 * SerialBuffer, quint8 BufferSize, quint8 * ConvertSize);
quint8 SaveConfig(quint8 * SerialBuffer, quint8 BufferSize, quint8 * ConvertSize);
quint8 FirmwareUpgrade(quint8 * SerialBuffer, quint8 BufferSize, quint8 * ConvertSize);
quint8 FirmwareVersion(quint8 * SerialBuffer, quint8 BufferSize, quint8 * ConvertSize);
quint8 GetErrorInfo(quint8 * SerialBuffer, quint8 BufferSize, quint8 * ConvertSize);

quint8 ParsingSerialInfo(SerialConfigInfo * info, quint8 * SerialBuffer, quint8 BufferSize);
quint8 ParsingCANInfo(CANConfigInfo * info, quint8 * SerialBuffer, quint8 BufferSize);
quint8 ParsingErrorInfo(CANErorrInfo * info, quint8 * SerialBuffer, quint8 BufferSize);